#include "include/types.h"

struct idt_entry_s{
   uint16 base_low;    // The lower 16 bits of the address to jump to when this interrupt occures: offset 0-15
   uint16 selector;      // Kernel segment selector in IDT
   uint8  zero;          // This must always be zero.
   uint8  flags;    //gate types, atttribute types etc.
   uint16 base_high;   // The upper 16 bits of the address to jump to. offset 16-31
} __attribute__((packed));

typedef struct idt_entry_s idt_entry_t;

struct idt_ptr_s_ {
    uint16 limit;
    uint32 base;     
} __attribute__((packed));
typedef struct idt_ptr_s_ idt_ptr_t;

struct gdt_entry_s
{
   uint16 limit_low;           // The lower 16 bits of the limit.
   uint16 base_low;            // The lower 16 bits of the base.
   uint8  base_middle;         // The next 8 bits of the base.
   uint8  access;              // Access flags, determine what ring this segment can be used in.
   uint8  granularity;         // eg: 4kb means 32 bit opcodes
   uint8  base_high;           // The last 8 bits of the base.
} __attribute__((packed));
typedef struct gdt_entry_s gdt_entry_t;

struct gdt_ptr_s_
{
   uint16 limit;               // The upper 16 bits of all selector limits.
   uint32 base;                // The address of the first gdt_entry_t struct.
}
 __attribute__((packed));
typedef struct gdt_ptr_s_ gdt_ptr_t; 

typedef struct registers_s
{
    unsigned int ds;
    unsigned int edi, esi, ebp, esp, ebx, edx, ecx, eax; //for popa
    unsigned int int_no, err_code;
    unsigned int eip, cs, eflags, useresp, ss;
} regs_t; 


#define _colours
#ifdef _colours

/**
00 Black       on Black
01 Blue        on Black
02 Green       on Black
03 Cyan        on Black
04 Red         on Black
05 Magenta     on Black
06 Brown       on Black
07 light Grey        on Black
08 dark Grey        on Black
09 Brt Blue    on Black
0A Brt Green   on Black
0B Brt Cyan    on Black
0C Brt Red     on Black
0D Brt Magenta on Black
0E Brt Yellow  on Black
0F Brt White   on Black
*/
#define _black 0x00
#define _blue 0x01
#define _green 0x02
#define _cyan 0x03
#define _red 0x04
#define _magneta 0x05
#define _brown 0x06
#define _brt_grey 0x07
#define _dark_grey 0x08
#define _brt_blue 0x09
#define _brt_green 0x0A
#define _brt_cyan 0x0B
#define _brt_red 0x0C
#define _brt_magneta 0x0D
#define _brt_yellow 0x0E
#define _brt_white 0x0F

#define _colour(frgd,bckg) ((bckg<<4)|frgd) 
#endif

#define sequence "/" 
#define _sequence '/' //character before the letter in an escape sequence

#define newline sequence"n"
#define htab sequence"t"
#define linefeed sequence"r"
#define bkspace sequence"b"

extern void mprintf(char *fmt,int colour);
extern void print_int(int a, int colour);
extern void print_int_(int a);
extern void secDelay(int secs);
void init_idt_gdt();
void print_regs(regs_t r);
extern uchar in(ushort _port);
extern void out(ushort _port, uchar _data);
#define outb(port,data) out(port,data)
#define outportb(port,data) out(port,data)
#define inb(port) in(port)
#define to_offset(x,y) ((y*80*2)+x)
#define screen_bcg_colour _colour(_brt_white,_black)

#define _keyboard
#define idt_gdt_proto
#define _irqs
#define _isrs
#define _idt
#define _gdt


#ifdef idt_gdt_proto

#ifdef _keyboard 

#ifndef keyboard_internal
extern void irq1_keyb(regs_t regs);
extern void init_keyboard();
#endif

#endif

#ifdef _idt
 typedef void (*isr_t)(regs_t);
 extern void idt_set_1();
 extern void idt_set_2(uint32);
  
 void *memset(void *s,int val,size_t size);
 static void init_idt();
 static void idt_set_gate(uint8 pos,uint32 base,uint16 sel,uint8 flags);
 void interrupt_handler(regs_t regs);
 void isr_handler(regs_t regs);
 void remap_irqs();
 void send_EOI(int num);
 void set_irq_handler(int pos, isr_t handler);
 void set_isr_handler(int pos, isr_t handler);
 
#endif

#ifdef _gdt
 extern void gdt_set_1();
 extern void gdt_set_2(uint32);
 
 void intit_idt_gdt();
 static void init_gdt();
 static void gdt_set_gate(int pos,uint32 base,uint32 limit,uint8 access,uint8 _granularity);
#endif

#define _isrs
#ifdef _isrs

  extern void _isr0();
  extern void _isr1();
  extern void _isr2();
  extern void _isr3();
  extern void _isr4();
  extern void _isr5();
  extern void _isr6();
  extern void _isr7();
  extern void _isr8();
  extern void _isr9();
  extern void _isr10();
  extern void _isr11();
  extern void _isr12();
  extern void _isr13();
  extern void _isr14();
  extern void _isr15();
  extern void _isr16();
  extern void _isr17();
  extern void _isr18();
  extern void _isr19();
  extern void _isr20();
  extern void _isr21();
  extern void _isr22();
  extern void _isr23();
  extern void _isr24();
  extern void _isr25();
  extern void _isr26();
  extern void _isr27();
  extern void _isr28();
  extern void _isr29();
  extern void _isr30();
  extern void _isr31();

#endif
#define _irqs
#ifdef _irqs
  extern void irq_0();
  extern void irq_1();
  extern void irq_2();
  extern void irq_3();
  extern void irq_4();
  extern void irq_5();
  extern void irq_6();
  extern void irq_7();
  extern void irq_8();
  extern void irq_9();
  extern void irq_10();
  extern void irq_11();
  extern void irq_12();
  extern void irq_13();
  extern void irq_14();
  extern void irq_15();

#endif

#endif

#define _timer
#ifdef _timer
  extern unsigned long timer_ticks;
  extern int time_cur_pos;
  
  extern void set_cursor_pos(int offset);
  extern void print_long_(unsigned long a);
  static void timer_handler(regs_t regs);
  void timer_handler_();
  void init_timer(uint32 frequency);
  
#endif

