#define ui
#define __types
#define idt_gdt

#ifdef __types 
#include "include/types.h"
#endif

#ifdef ui
#define _keyboard
#define _mprintf
#define text_mode
#define _welcome
#define _colours
#endif



#define name "ManuOS"

#define outportb(port,data) out(port,data)
#define inportb(port) in(port)
#define outb(port,data) out(port,data)
#define inb(port) in(port)

#define fp_seg(fp) (*((unsigned *)&(fp) + 1))
#define fp_off(fp) (*((unsigned *)&(fp)))


#define sequence "/" 
#define _sequence '/' //character before the letter in an escape sequence

#define newline sequence"n"
#define htab sequence"t"
#define linefeed sequence"r"
#define bkspace sequence"b"



#ifdef _colours

/**
00 Black       on Black
01 Blue        on Black
02 Green       on Black
03 Cyan        on Black
04 Red         on Black
05 Magenta     on Black
06 Brown       on Black
07 light Grey        on Black
08 dark Grey        on Black
09 Brt Blue    on Black
0A Brt Green   on Black
0B Brt Cyan    on Black
0C Brt Red     on Black
0D Brt Magenta on Black
0E Brt Yellow  on Black
0F Brt White   on Black
*/
#define _black 0x00
#define _blue 0x01
#define _green 0x02
#define _cyan 0x03
#define _red 0x04
#define _magneta 0x05
#define _brown 0x06
#define _brt_grey 0x07
#define _dark_grey 0x08
#define _brt_blue 0x09
#define _brt_green 0x0A
#define _brt_cyan 0x0B
#define _brt_red 0x0C
#define _brt_magneta 0x0D
#define _brt_yellow 0x0E
#define _brt_white 0x0F

#define _colour(frgd,bckg) ((bckg<<4)|frgd) 
#endif

#define low_byte(n) (n & 0x00ff)
#define high_byte(n) ((n>>8)&0x00ff)

#ifdef text_mode

#define screen_size (80*50)
#define screen_bcg_colour _colour(_brt_white,_black)

#define special_text_colour _colour(_green,_black)
#define error_text_colour _colour(_red,_black)
#define success_text_colour _colour(_blue,_black)
#define text_colour screen_bcg_colour

#define line_start(pos) ((pos- (pos%80) ))
#define vdu_mem 0xB8000
#define to_offset(x,y) (y*cols*2+x)

#define _1sec 1
#define _1min 60
#define _1hour 3600
#define to_bcgoffset(offset) \
 if(offset%2==0)\
  offset+=1; 

    
#define centre_pos(aa,a) ((aa/2)-(a/2))
#define cols 80
#define rows 50
#define _4KB 4096



#ifdef _welcome
#define bckg _brt_grey
#define welcome_scr_bcg _colour(_black,_black)
#define window_colour   _colour(_green,bckg)
#define dots_colour_1   _colour(_brt_grey,bckg)
#define dots_colour_2   _colour(_red,bckg)
#define dots_colour     _colour(_red,bckg)
void welcome();
void print_manuos();
void terminal(); 

#endif

#ifdef ui

void cls(int colour);
void update_cursor(uint16 position);
void set_cursor0(void);
void set_cursor(int offset);
void move_cursor(int distance);
int get_cursor_pos();
void set_cursor0(void);
void set_cursor_pos(int offset);

void draw_window(int startx,int starty,int endx,int endy,int colour);
void set_text_colour(int colour, int from,int to);
void dots_countdown(int till);
int circular_dots(char *a,int millisecs);

int linear_dots(char *a,int millisecs);
void scroll_up(uint16 lines, uint16 pos, uint16 diff);
void scroll_down(uint8 lines,uint16 pos);
void print_time();


//void outc_bcg(int colour);
int outbyte(int c);
int outpix(int c,int colour);
int outpix_pos(int c,int colour,int offset);
void outc_bcg_pos(int colour, int x, int y);
void _outc_bcg_pos(int colour, int offset);

#endif

#ifdef _mprintf
void mprintf(char *fmt,int colour);
#endif

int strlen(char *str);
int strcmp(char *str1,char *str2);
double power(double a,int t);
int intlen(int a);
int intat(int a, int pos);
void itostr(int a,char *aa);
void print_int(int a, int colour);
void print_int_(int a);
void print_long_(unsigned long a);

uchar in(ushort _port);
void out(ushort _port, uchar _data);


void Delay(long secs);
void countdown_echo(int till,int colour);

void secDelay(int secs);
void milliDelay(long millisecs);
void microDelay(unsigned long microsecs);
//void tinydelay(unsigned long a);

#endif

void stop_beep();
void start_beep();
void pixtry();
void asm_try();
extern void init_idt_gdt();
extern void init_timer(uint32 frequency);

